/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Map/mapelt.h>
#include <MapServ/maphandler.h>
#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/mapcomps.h>

#include <Vpf/vpfcov.h>
#include <Vpf/vpfdb.h>
#include <Vpf/vpffclass.h>
#include <Vpf/vpflib.h>

#include <VpfUnidraw/vpfclasses.h>
#include <VpfUnidraw/vpfcmds.h>
#include <VpfUnidraw/vpfcomps.h>
#include <VpfUnidraw/vpfeditor.h>
#include <VpfUnidraw/vpffunc.h>
#include <VpfUnidraw/vpfviewstate.h>

#include <MapUnidraw/mapcmds.h>
#include <MapUnidraw/mapinspect.h>
#include <MapUnidraw/mapproj.h>
#include <MapUnidraw/mapstatecmds.h>
#include <MapUnidraw/mapstates.h>

#include <Attribute/attribute.h>
#include <Attribute/attrlist.h>
#include <Attribute/attrvalue.h>

#include <OverlayUnidraw/ovdoer.h>
#include <OverlayUnidraw/ovpage.h>
#include <OverlayUnidraw/ovviewer.h>
#include <UniIdraw/idvars.h>

#include <Unidraw/catalog.h>
#include <Unidraw/clipboard.h>
#include <Unidraw/ctrlinfo.h>
#include <Unidraw/iterator.h>
#include <Unidraw/keymap.h>
#include <Unidraw/kybd.h>
#include <Unidraw/selection.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/Commands/edit.h>
#include <Unidraw/Graphic/ellipses.h>
#include <Unidraw/Graphic/lines.h>
#include <Unidraw/Graphic/polygons.h>

#include <InterViews/border.h>
#include <InterViews/box.h>
#include <InterViews/cursor.h>
#include <InterViews/frame.h>
#include <InterViews/glue.h>
#include <InterViews/menu.h>
#include <InterViews/message.h>
#include <InterViews/panner.h>
#include <InterViews/tray.h>
#include <InterViews/window.h>

#include <stdlib.h>
#include <string.h>

/*****************************************************************************/

VpfEditor::VpfEditor(OverlayComp* gc, OverlayKit* ok) 
: MapEditor(false, ok) {
    _projection = nil;
    _mapcomp = nil;
    _tiles = nil;
    _graticule = nil;
    Catalog* catalog = unidraw->GetCatalog();
    char* vpfdir = strdup(catalog->GetAttribute("database"));
    SetMapDatabase(new VpfDatabase(vpfdir));

    Init(gc, "VpfEditor");
    MapProjectionCmd* pcmd = new MapProjectionCmd(this);
    pcmd->Default();

    MapDatabase* mapdb = GetMapDatabase();
    MapLibrary* maplib = (MapLibrary*)mapdb->library(_libstate->maplib());
    if (!maplib) {

    }
    else {
    VpfExtent* ext = ((VpfLibrary*)maplib)->extent();
    VpfViewState* view = new VpfViewState(GetProjection(), ext->x1, ext->y1);
    SetMapViewState(view);
    }
    Initializing(true);
}

VpfEditor::VpfEditor(boolean initflag, OverlayKit* ok) 
: MapEditor(initflag, ok) {
    _graticule = nil;
}

VpfEditor::VpfEditor(const char* file, OverlayKit* ok)
: MapEditor(false, ok)
{
    _projection = nil;
    _mapcomp = nil;
    _tiles = nil;
    _graticule = nil;
    //MapProjectionCmd* pcmd = new MapProjectionCmd(this);
    //pcmd->Default();
    if (file == nil) {
	Init();
    MapProjectionCmd* pcmd = new MapProjectionCmd(this);
    pcmd->Default();

    } else {
	Catalog* catalog = unidraw->GetCatalog();
	OverlayComp* comp;

	if (catalog->Retrieve(file, (Component*&) comp)) {
	  Iterator it;
	  comp->First(it);
	  GraphicComp* dbcomp = comp->GetComp(it);
	  if (dbcomp && dbcomp->IsA(MAPDB_COMP)) {
	    VpfDatabase* db = ((VpfDbComp*)dbcomp)->GetDatabase();
	    SetMapDatabase(db);
	    //db->init();
	    SetMapDbComp((VpfDbComp*)dbcomp);
	  Init(comp);
    MapProjectionCmd* pcmd = new MapProjectionCmd(this);
    pcmd->Default();
    //	    OpenInit();
	  }
	  else {
    MapProjectionCmd* pcmd = new MapProjectionCmd(this);
    pcmd->Default();
	  
	  Init(comp);
	  }
	} else {
	    Init();
    MapProjectionCmd* pcmd = new MapProjectionCmd(this);
    pcmd->Default();
	    fprintf(stderr, "vpfviewer: couldn't open %s\n", file);
	}
    }
}

void VpfEditor::Init (OverlayComp* comp, const char* name) {
  _curr_other = _prev_other = 0;
  _texteditor = nil;
  if (!comp) comp = new VpfIdrawComp();
  _overlay_kit->Init(comp, name);
  InitFrame();
}

void VpfEditor::InitCommands() {
  if (!_mapdb) {
    Catalog* catalog = unidraw->GetCatalog();
    char* mag = strdup(catalog->GetAttribute("initmag"));
    double fmag = atof(mag);
    char* libname = strdup(catalog->GetAttribute("library"));
    libstate()->maplib(libname);
    char* covname = strdup(catalog->GetAttribute("coverage"));
    covstate()->mapcov(covname, "");
    char* fcname = strdup(catalog->GetAttribute("fclass"));
    fclassstate()->mapfclass(fcname, "");
    Initializing(true);
    MapShowCmd* show = new MapShowCmd(this);
    show->SetMag(fmag);
    CommandPusher* pusher = new CommandPusher(show);
    pusher->Push();
  }
  else {
    MapShowAllCmd* showall = new MapShowAllCmd(this);

    Attribute* imattr;
    if (imattr = ((OverlayComp*)GetGraphicComp())->GetAttributeList()->GetAttr("initmag")) {
      showall->SetMag(imattr->Value()->float_val());
    }
    else {
    Catalog* catalog = unidraw->GetCatalog();
    char* mag = strdup(catalog->GetAttribute("initmag"));
    double fmag = atof(mag);
    showall->SetMag(fmag);
    }
    //showall->Execute();
    Initializing(true);
    CommandPusher* pusher = new CommandPusher(showall);
    pusher->Push();

    Attribute* lonattr;
    Attribute* latattr;
    if ((lonattr = ((OverlayComp*)GetGraphicComp())->GetAttributeList()->GetAttr("initlon")) &&
	(latattr = ((OverlayComp*)GetGraphicComp())->GetAttributeList()->GetAttr("initlat"))) {
      MapCenterCmd* cencmd = new MapCenterCmd(this);
      cencmd->CenterLonLat(lonattr->Value()->float_val(),
			   latattr->Value()->float_val());
      CommandPusher* pusher2 = new CommandPusher(cencmd);
      pusher2->Push();
      ZoomCmd* zcmd = new ZoomCmd(this, 4.0);
      CommandPusher* pusher3 = new CommandPusher(zcmd);
      pusher3->Push();
    }
  }
  MapUpdateMenusCmd* umcmd = new MapUpdateMenusCmd(this);
  CommandPusher* pusher4 = new CommandPusher(umcmd);
  pusher4->Push();
}

void VpfEditor::tiles(Selection* s) {
    _tiles = s;
    GetViewer()->GetSelection()->Clear();
    GetViewer()->GetSelection()->Exclusive(s);
}

VpfDatabase* VpfEditor::GetMapDatabase() {
    return (VpfDatabase*)_mapdb;
}

void VpfEditor::SetMapDatabase(VpfDatabase* db) {
    _mapdb = db;
}

void VpfEditor::InitMapComp() {
  if (!_mapdb) {
    Catalog* catalog = unidraw->GetCatalog();
    char* dbpath = strdup(catalog->GetAttribute("database"));
    if (strlen(dbpath)>0)
      _mapdb = new VpfDatabase(dbpath);
  }
    _mapcomp = new VpfDbComp((VpfDatabase*)_mapdb);
    Clipboard* clb = new Clipboard();
    clb->Append(_mapcomp);
    PasteCmd* pcmd = new PasteCmd(this, clb);
    pcmd->Execute();
    Selection* s = GetViewer()->GetSelection();
    s->Clear();
}

void VpfEditor::FClassState(VpfFeatureClass* fc, boolean val) {
    ((VpfKit*)_overlay_kit)->FClassState(fc, val);
}

void VpfEditor::toolbar0() {
     ((VpfKit*)_overlay_kit)->toolbar0();
}

void VpfEditor::toolbar1() {
     ((VpfKit*)_overlay_kit)->toolbar1();
}

void VpfEditor::ExplainTileSel(char* libname) {
    ((VpfKit*)_overlay_kit)->ExplainTileSel(libname);
}

void VpfEditor::UpdateLibraryMenu() {
     ((VpfKit*)_overlay_kit)->UpdateLibraryMenu();
}

void VpfEditor::UpdateLayerMenu() {
     ((VpfKit*)_overlay_kit)->UpdateLayerMenu();
}

void VpfEditor::AddCommands(ComTerp* comterp) {

    comterp->add_command("loadvpffc", new LoadVpfFCFunc(comterp));

    MapservHandler::AddCommands(comterp);
}
