/*
 * Copyright (c) 1997,1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <VpfUnidraw/vpfcomps.h>
#include <VpfUnidraw/vpfcreator.h>
#include <VpfUnidraw/vpffunc.h>
#include <MapUnidraw/mapcmds.h>
#include <Vpf/vpfdb.h>
#include <Vpf/vpflib.h>
#include <Vpf/vpfcov.h>
#include <Vpf/vpffclass.h>
#include <Vpf/vpffeature.h>

/*****************************************************************************/

#ifdef VPF_FREE /* if it is not already pulled in by MapServ */

LoadVpfFCFunc::LoadVpfFCFunc(ComTerp* comterp) : ComFunc(comterp) {
  _tileid_symid = symbol_add("tileid");
}

void LoadVpfFCFunc::execute() {
    ComValue dbpath(stack_arg(0));
    ComValue libname(stack_arg(1));
    ComValue covname(stack_arg(2));
    ComValue fcname(stack_arg(3));
    ComValue tileid(stack_key(_tileid_symid));
    reset_stack();
    ComValue result;

    Creator::instance(new VpfCreator());
    const char* dbdir = dbpath.string_ptr();
    VpfDatabase* db = new VpfDatabase(dbdir);
    VpfLibrary* lib = db->library(libname.string_ptr());
    VpfDbComp* dbcomp = new VpfDbComp(db);
    MapShowCmd* showcmd = new MapShowCmd();
    showcmd->SetUseEditor(false);
    showcmd->SetLibName(libname.string_ptr());
    showcmd->SetCovName(covname.string_ptr());
    showcmd->SetFClassName(fcname.string_ptr());
    VpfExtent* ext = lib->extent();
    char lon0[80];
    sprintf(lon0, "lon_0=%f", ext->x1);  // let proj pick the utm zone
    char* parms[4];
    parms[0] = "proj=utm";
    parms[1] = "ellps=WGS84";
    parms[2] = lon0;
    parms[3] = "no_defs";
    PJ* proj = pj_init(4, parms);
    showcmd->SetProjection(proj);
    if (tileid.type() != ComValue::UnknownType) {
      int tid = tileid.int_val();
      showcmd->SetTileId(tid);
    }
    dbcomp->Interpret(showcmd);

    VpfLibComp* libcomp = (VpfLibComp*)dbcomp->FindLibComp(lib);
    VpfCoverage* cov = lib->coverage(covname.string_ptr());
    VpfCovComp* covcomp = (VpfCovComp*)libcomp->FindCovComp(cov);
    VpfFeatureClass* fclass = cov->feature_class(fcname.string_ptr());
    VpfFClassComp* fccomp = (VpfFClassComp*)covcomp->FindFClassComp(fclass);
    result.type(AttributeValue::ObjectType);
    result.obj_ref() = fccomp;
    push_stack(result);
}

#endif /* VPF_FREE */
